"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import json
from pacifico_devel.api.src.util import Dates, Enumerations, AmazonWebServices

def DataPointReportFromMessageReport(reports, awsAccessKey, awsSecretKey, author='BFI', versionName='', largeOutputs=True):
    if largeOutputs:
        if len(reports) > 10000:  # If it's too big, the compression will be done locally
            if author == 'BFI':
                author = 'Pacifico'
            if versionName == '':
                authorKey = author
            else:
                authorKey = author + '-' + versionName
            return json.dumps([authorKey] + [report.getJson(False) for report in reports])
    allDataPointReports = {}
    for report in reports:
        dataPointValues = {}
        # author = dataPointList[18]
        # versionName = dataPointList[17]
        datePublication = Dates.getDateString(report.getDatepublication())
        # dataPointValues['datePublication'] = datePublication
        if report.getFixingPublication() != Enumerations.Fixing.EOD.value:
            fixing = Enumerations.Fixing.getString(report.getFixingPublication())
            dataPointValues['fixing'] = fixing
        dateEffective = Dates.getDateTimeString(report.getDateEffective())
        # dataPointValues['dateEffective'] = dateEffective
        document = report.getDocument()
        chapter = report.getChapter()
        section = report.getSection()
        subsection = report.getSubsection()
        paragraph = report.getParagraph()
        item = report.getItem()
        variant = report.getVariant()
        variantValueType = variant.getValueType()  # Enumerations.ValueType(dataPointList[8])
        variantValue = variantValueType.fromDB(variant.getValue(asString=True))
        try:
            if variantValueType == Enumerations.ValueType.ValueType_File:
                fileName = variantValue.split('.amazonaws.com/', 1)[1]
                bucketName = variantValue.split('.s3.')[0].replace('https://', '')
                secondsItLasts = 86400  # 24hrs
                variantValue = AmazonWebServices.AmazonWebServices.getUrl(fileName, secondsItLasts, awsAccessKey, awsSecretKey, bucketName)
        except:
            pass
        variantValueType = variantValueType.getString()
        dataPointValues['variant'] = {'value': variantValue, 'type': variantValueType}
        #if dataPointList[9] not in [0, 1]:
        #    dataPointValues['quality'] = Enumerations.Quality.getString(Enumerations.Quality(dataPointList[9]))
        dateTenor = Dates.getDateTimeString(report.getDateTenor())
        if dateTenor != dateEffective:
            dataPointValues['dateTenor'] = dateTenor
        other = report.getComment()
        if other != '':
            dataPointValues['other'] = other
        addDataPointReports(author, versionName, datePublication, dateEffective, document, chapter, section, subsection, paragraph, item, dataPointValues, allDataPointReports)
    data = json.dumps(allDataPointReports)
    return data

def DataPointReportFromDB(dataPointsLists, awsAccessKey, awsSecretKey):
    allDataPointReports = {}
    for dataPointList in dataPointsLists:
        dataPointValues = {}
        author = dataPointList[18]
        versionName = dataPointList[17]
        datePublication = Dates.getDateString(Dates.julianToDate(float(dataPointList[9])))
        # dataPointValues['datePublication'] = datePublication
        if dataPointList[10] != Enumerations.Fixing.EOD.value:
            fixing = Enumerations.Fixing.getString(Enumerations.Fixing(dataPointList[10]))
            dataPointValues['fixing'] = fixing
        dateEffective = Dates.getDateTimeString(Dates.julianToDateTime(float(dataPointList[11])))
        # dataPointValues['dateEffective'] = dateEffective
        document = dataPointList[1]
        chapter = dataPointList[2]
        section = dataPointList[3]
        subsection = dataPointList[4]
        paragraph = dataPointList[5]
        item = dataPointList[6]
        variantValueType = Enumerations.ValueType(dataPointList[8])
        variantValue = variantValueType.fromDB(dataPointList[7])
        if variantValueType == Enumerations.ValueType.ValueType_File:
            fileName = variantValue.split('.amazonaws.com/', 1)[1]
            bucketName = variantValue.split('.s3.')[0].replace('https://', '')
            secondsItLasts = 86400  # 24hrs
            variantValue = AmazonWebServices.AmazonWebServices.getUrl(fileName, secondsItLasts, awsAccessKey, awsSecretKey, bucketName)
        variantValueType = variantValueType.getString()
        dataPointValues['variant'] = {'value': variantValue, 'type': variantValueType}
        #if dataPointList[9] not in [0, 1]:
        #    dataPointValues['quality'] = Enumerations.Quality.getString(Enumerations.Quality(dataPointList[9]))
        dateTenor = Dates.getDateTimeString(Dates.julianToDateTime(float(dataPointList[12])))
        if dateTenor != dateEffective:
            dataPointValues['dateTenor'] = dateTenor
        other = dataPointList[19]
        if other != '':
            dataPointValues['other'] = other
        addDataPointReports(author, versionName, datePublication, dateEffective, document, chapter, section, subsection, paragraph, item, dataPointValues, allDataPointReports)
    data = json.dumps(allDataPointReports)
    return data

def addDataPointReports(author, versionName, datePublication, dateEffective, document, chapter, section, subsection, paragraph, item, dataPointValues, allDataPointReports):
    if author == 'BFI':
        author = 'Pacifico'
    if versionName == '':
        authorKey = author
    else:
        authorKey = author + '-' + versionName
    if authorKey in allDataPointReports.keys():
        if datePublication in allDataPointReports[authorKey].keys():
            if dateEffective in allDataPointReports[authorKey][datePublication].keys():
                if document in allDataPointReports[authorKey][datePublication][dateEffective].keys():
                    if chapter in allDataPointReports[authorKey][datePublication][dateEffective][document].keys():
                        if section in allDataPointReports[authorKey][datePublication][dateEffective][document][chapter].keys():
                            if subsection in allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section].keys():
                                if paragraph in allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection].keys():
                                    if item in allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection][paragraph].keys():
                                        if dataPointValues in allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection][paragraph][item]:
                                            print('ioDataPointReport: This should never happen.')
                                        else:
                                            allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection][paragraph][item].append(dataPointValues)
                                    else:
                                        allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection][paragraph].update({item: [dataPointValues]})
                                else:
                                    allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section][subsection].update({paragraph: {item: [dataPointValues]}})
                            else:
                                allDataPointReports[authorKey][datePublication][dateEffective][document][chapter][section].update({subsection: {paragraph: {item: [dataPointValues]}}})
                        else:
                            allDataPointReports[authorKey][datePublication][dateEffective][document][chapter].update({section: {subsection: {paragraph: {item: [dataPointValues]}}}})
                    else:
                        allDataPointReports[authorKey][datePublication][dateEffective][document].update({chapter: {section: {subsection: {paragraph: {item: [dataPointValues]}}}}})
                else:
                    allDataPointReports[authorKey][datePublication][dateEffective].update({document: {chapter: {section: {subsection: {paragraph: {item: [dataPointValues]}}}}}})
            else:
                allDataPointReports[authorKey][datePublication].update({dateEffective: {document: {chapter: {section: {subsection: {paragraph: {item: [dataPointValues]}}}}}}})
        else:
            allDataPointReports[authorKey].update({datePublication: {dateEffective: {document: {chapter: {section: {subsection: {paragraph: {item: [dataPointValues]}}}}}}}})
    else:
        allDataPointReports.update({authorKey: {datePublication: {dateEffective: {document: {chapter: {section: {subsection: {paragraph: {item: [dataPointValues]}}}}}}}}})

def _recursive__addDataPointReports(author, versionName, document, chapter, section, subsection, paragraph, item, dataPointValues, allDataPointReports):
    if versionName == '':
        authorKey = author
    else:
        authorKey = author + '-' + versionName
    if authorKey in allDataPointReports.keys():
        if document in allDataPointReports[authorKey].keys():
            if chapter in allDataPointReports[authorKey][document].keys():
                if section in allDataPointReports[authorKey][document][chapter].keys():
                    if subsection in allDataPointReports[authorKey][document][chapter][section].keys():
                        if paragraph in allDataPointReports[authorKey][document][chapter][section][subsection].keys():
                            if item in allDataPointReports[authorKey][document][chapter][section][subsection][paragraph].keys():
                                print('ioDataPointReport: This should never happen.')
                            else:
                                allDataPointReports[authorKey][document][chapter][section][subsection][paragraph].update({item: dataPointValues})
                        else:
                            if paragraph != '':
                                allDataPointReports[authorKey][document][chapter][section][subsection].update({paragraph: {item: dataPointValues}})
                            else:
                                allDataPointReports[authorKey][document][chapter][section][subsection].update({item: dataPointValues})
                    else:
                        if subsection != '':
                            if paragraph != '':
                                allDataPointReports[authorKey][document][chapter][section].update({subsection: {paragraph: {item: dataPointValues}}})
                            else:
                                allDataPointReports[authorKey][document][chapter][section].update({subsection: {item: dataPointValues}})
                        else:
                            allDataPointReports[authorKey][document][chapter][section].update({item: dataPointValues})
                else:
                    if section != '':
                        if subsection != '':
                            if paragraph != '':
                                allDataPointReports[authorKey][document][chapter].update({section: {subsection: {paragraph: {item: dataPointValues}}}})
                            else:
                                allDataPointReports[authorKey][document][chapter].update({section: {subsection: {item: dataPointValues}}})
                        else:
                            allDataPointReports[authorKey][document][chapter].update({section: {item: dataPointValues}})
                    else:
                        allDataPointReports[authorKey][document][chapter].update({item: dataPointValues})
            else:
                if chapter != '':
                    if section != '':
                        if subsection != '':
                            if paragraph != '':
                                allDataPointReports[authorKey][document].update({chapter: {section: {subsection: {paragraph: {item: dataPointValues}}}}})
                            else:
                                allDataPointReports[authorKey][document].update({chapter: {section: {subsection: {item: dataPointValues}}}})
                        else:
                            allDataPointReports[authorKey][document].update({chapter: {section: {item: dataPointValues}}})
                    else:
                        allDataPointReports[authorKey][document].update({chapter: {item: dataPointValues}})
                else:
                    allDataPointReports[authorKey][document].update({item: dataPointValues})
        else:
            if chapter != '':
                if section != '':
                    if subsection != '':
                        if paragraph != '':
                            allDataPointReports[authorKey].update({document: {chapter: {section: {subsection: {paragraph: {item: dataPointValues}}}}}})
                        else:
                            allDataPointReports[authorKey].update({document: {chapter: {section: {subsection: {item: dataPointValues}}}}})
                    else:
                        allDataPointReports[authorKey].update({document: {chapter: {section: {item: dataPointValues}}}})
                else:
                    allDataPointReports[authorKey].update({document: {chapter: {item: dataPointValues}}})
            else:
                allDataPointReports[authorKey].update({document: {item: dataPointValues}})
    else:
        if chapter != '':
            if section != '':
                if subsection != '':
                    if paragraph != '':
                        allDataPointReports.update({authorKey: {document: {chapter: {section: {subsection: {paragraph: {item: dataPointValues}}}}}}})
                    else:
                        allDataPointReports.update({authorKey: {document: {chapter: {section: {subsection: {item: dataPointValues}}}}}})
                else:
                    allDataPointReports.update({authorKey: {document: {chapter: {section: {item: dataPointValues}}}}})
            else:
                allDataPointReports.update({authorKey: {document: {chapter: {item: dataPointValues}}}})
        else:
            allDataPointReports.update({authorKey: {document: {item: dataPointValues}}})


import pandas as pd
import typing as typ
import pathlib as pth
import dataframe_image as dfi
import pandas.io.formats.style as pdst
import numpy as np
import matplotlib as mpl
from matplotlib.colors import ListedColormap, LinearSegmentedColormap

import pacifico_devel.util.reportTemplate.src.core.Plot.Plot as bplt
from pacifico_devel.util.reportTemplate.src.util.Patches import ScreenshotPatch
import pacifico_devel.util.reportTemplate.src.core.Style.Style as stl
import pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors as pc

import pacifico_devel.util.lexikon.src.util.Enumerations as lenm
import pacifico_devel.util.lexikon.src.core.Translator as trl


# TO DO: Implement formatter

class PlotTableOneColorMap(bplt.Plot):
    def __init__(self,
                 df: pd.DataFrame,
                 languageInput: lenm.Language,
                 languageOutput: lenm.Language,
                 shape: typ.Tuple[float, float],
                 basePlotResolution: int,
                 style: typ.Optional[stl.Style] = None,
                 formatter: typ.Optional[typ.Dict[str, str]] = None):
        super(PlotTableOneColorMap, self).__init__(df=df,
                                                   shape=shape,
                                                   languageInput=languageInput,
                                                   languageOutput=languageOutput,
                                                   basePlotResolution=basePlotResolution,
                                                   style=style)
        self._formatter = formatter

    def saveImage(self) -> None:
        """


        Args:
            dfTranslated:

        Returns:

        """
        chart = self._makeChart()
        ScreenshotPatch.patch()  # Monkey patch for chrome screenshots server
        print(f"SAVING TABLE TO: {self.pathImage} ...")
        dfi.export(obj=chart,
                   filename=str(self.pathImage))
        print(f"SAVING TABLE TO: {self.pathImage} ... DONE!")

    def _makeChart(self) -> pdst.Styler:
        # dfAux = self._translateDataFrame(df)
        thProps = [("background-color", pc.PacificoPlotColors.BLUE.getCodeString())]

        tdProps = [('font-family', 'Klartext Mono'),
                   ("color", pc.PacificoPlotColors.BLUE.getCodeString()),
                   ('text-align', 'center'),
                   ('font-size', '12pt')]

        index_col = {'selector': 'th.col_heading',
                     'props': [('background-color', pc.PacificoPlotColors.BLUE.getCodeString()),
                               ('font-family', 'Klartext Mono'),
                               ('color', 'white'),
                               #('border-bottom', '3px solid #ffffff'),
                               ('text-align', 'center'),
                               ('font-size', '12pt')]}

        indexName = {'selector': 'th.blank',
                     'props': [('background-color', pc.PacificoPlotColors.BLUE.getCodeString())]}
        #('border-bottom', '3px solid #ffffff')

        splitCol = {'selector': '.col0',
                    'props': [('border-right', '5px solid #ffffff')]}

        index = {'selector': 'th.row_heading',
                 'props': [('background-color', pc.PacificoPlotColors.BLUE.getCodeString()),
                           ('font-family', 'Klartext Mono'),
                           ('color', 'white'),
                           #('border-right', '3px solid #ffffff'),
                           ('font-weight', 'bold')]}

        dfst = self.df.style.set_table_styles([{"selector": "th", "props": thProps},
                                               {"selector": "td.data", "props": tdProps},
                                               index,
                                               indexName,
                                               index_col,
                                               splitCol
                                               ]).hide_index()

        cmap = PlotTableOneColorMap.__makeColormap()
        dfst = dfst.background_gradient(cmap=cmap,
                                        axis=None
                                        ).highlight_null('white')

        if self._formatter is None:
            dfst = dfst.format(lambda x: "{:.2f}".format(x) if isinstance(x, float) else '{}'.format(x), na_rep='')
        else:
            dfst = dfst.format(self._formatter, na_rep='')
        return dfst

    @staticmethod
    def __colorFader(c1, c2, mix: float = 0):  # fade (linear interpolate) from color c1 (at mix=0) to c2 (mix=1)
        c1 = np.array(mpl.colors.to_rgb(c1))
        c2 = np.array(mpl.colors.to_rgb(c2))
        return mpl.colors.to_hex((1 - mix) * c1 + mix * c2)

    @staticmethod
    def __makeColormap():
        color1 = []
        for i in range(1, 100 + 1):
            color1.append(
                PlotTableOneColorMap.__colorFader(pc.PacificoPlotColors.PINK.getCodeString(), "#ffffff", i / 100))
        color1 = color1[0:90]
        return ListedColormap(color1)

    def df_to_excel(self, writer, tag):
        """
        saves DataFrame to excel
        Returns:

        """

        self.df.to_excel(writer, sheet_name=tag)
    # @staticmethod
    # def make_gradient(value, min_value: float, max_value: float, cmap1, cmap2):
    #     """
    #     Parameters
    #     ----------
    #
    #     v: tuple of (word, length)
    #     min_length: int
    #         minimum length of all words in the matrix
    #     max_length: int
    #         maximum length of all words in the matrix
    #     cmap: matplotlib color map, default value here is 'YlGn'
    #
    #     Returns
    #     -------
    #
    #     string:
    #         CSS setting a colour
    #
    #     For Matplotlib colormaps:
    #     See: https://matplotlib.org/stable/tutorials/colors/colormaps.html
    # """
    #     if isinstance(value, float):
    #         if value <= 0:
    #             max_value = 0
    #             value = (value - min_value) / (max_value - min_value)
    #             rgba = cmap1(value)
    #         else:
    #             min_value = 0
    #             value = (value - min_value) / (max_value - min_value)
    #             rgba = cmap2(value)
    #         return f'background-color: {mpl.colors.rgb2hex(rgba)};'





"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import json
import datetime
from pacifico.util import Enumerations, Dates


class Variant:
    def __init__(self, value, valueType=Enumerations.ValueType.ValueType_Double):
        if not isinstance(value, valueType.getType()):
            if isinstance(value, str):
                if self.getValueType() == Enumerations.ValueType.ValueType_Bool:
                    value = bool(value)
                elif self.getValueType() == Enumerations.ValueType.ValueType_Date:
                    value = Dates.julianToDate(float(value))
                elif self.getValueType() == Enumerations.ValueType.ValueType_DateTime:
                    value = Dates.julianToDateTime(float(value))
                elif self.getValueType() == Enumerations.ValueType.ValueType_Integer:
                    value = int(value)
                elif self.getValueType() == Enumerations.ValueType.ValueType_Double:
                    value = float(value)
                else:
                    raise TypeError("The value and the valueType of a Variant object must be coherent.")
            else:
                raise TypeError("The value and the valueType of a Variant object must be coherent.")
        self.value = value
        self.valueType = valueType

    # valueType
    def _get_valueType(self):
        return self.__valueType
    def _set_valueType(self, value):
        if not isinstance(value, Enumerations.ValueType):
            raise TypeError("The valueType must be set to a class 'Enumerations.ValueType'.")
        self.__valueType = value
    valueType = property(_get_valueType, _set_valueType)

    def getValue(self, asString=False):
        if asString:
            if self.getValueType() == Enumerations.ValueType.ValueType_Bool:
                return str(self.value)
            elif self.getValueType() == Enumerations.ValueType.ValueType_Date:
                return str(Dates.dateToJulian(self.value))
            elif self.getValueType() == Enumerations.ValueType.ValueType_DateTime:
                return str(Dates.dateTimeToJulian(self.value))
            elif self.getValueType() == Enumerations.ValueType.ValueType_Integer or self.getValueType() == Enumerations.ValueType.ValueType_Double:
                return str(self.value)
            else:
                return self.value
        else:
            return self.value

    def getValueType(self):
        return self.valueType

    @staticmethod
    def __serializeAndClean(dictionary):
        delete = []
        for key, value in zip(dictionary.keys(), dictionary.values()):
            if value == None:
                delete.append(key)
            elif isinstance(value, Enumerations.Enum):
                dictionary[key] = value.value
            elif isinstance(value, (datetime.date, datetime.datetime)):
                dictionary[key] = value.isoformat()
            elif isinstance(value, bool):
                dictionary[key] = int(value)
            else:
                dictionary[key] = value
        for key in delete:
            del dictionary[key]
        return dictionary

    def getJson(self):
        # Get all attributes as a dictionary
        data = self.__dict__.copy()
        # Delete object private prefix from attribute names
        oldKeys = list(data.keys())
        for key in oldKeys:
            newKey = key.replace('_' + self.__class__.__name__ + '__', '')
            data[newKey] = data.pop(key)
        # Serialize object within the attribute and delete empty attributes
        data = self.__serializeAndClean(data)
        # jsonize data
        dataJson = json.dumps(data)
        return dataJson

    @staticmethod
    def fromJson(jsonString):
        if isinstance(jsonString, str):
            variantDict = json.loads(jsonString)
        else:
            variantDict = jsonString
        if 'valueType' in variantDict.keys():
            valueType = Enumerations.ValueType(variantDict['valueType'])
        elif 'type' in variantDict.keys():
            valueType = Enumerations.ValueType(variantDict['type'])
        else:
            valueType = Enumerations.ValueType.ValueType_String
        value = valueType.fromJson(variantDict['value'])
        variant = Variant(value, valueType)
        return variant

    @staticmethod
    def getEmptyVariant():
        return Variant('', Enumerations.ValueType.ValueType_String)

if __name__ == '__main__':
    x = Variant(datetime.date(2020, 1, 1), Enumerations.ValueType.ValueType_Date)
    y = x.getJson()
    print(y)
    print(Variant.fromJson(y).getValueType())
    z = Variant(True, Enumerations.ValueType.ValueType_Bool)
    valueAsString = z.getValue(asString=True)
    print(valueAsString, type(valueAsString))


"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import boto3
from ...util.aws import Enumerations
from ...util.cfg import Configuration


def getToken(tokenId, awsAccessKey, awsSecretKey):
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    response = client.get_api_key(apiKey=tokenId, includeValue=True)
    token = response['value']
    return token

def getName(tokenId, awsAccessKey, awsSecretKey):
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    response = client.get_api_key(apiKey=tokenId, includeValue=True)
    token = response['name']
    return token

def createToken(name, awsAccessKey, awsSecretKey):
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    response = client.create_api_key(name=name, enabled=True)
    return response

def __applySingleAction(tokenId, usagePlan, action, awsAccessKey, awsSecretKey, connection):
    if not isinstance(usagePlan, Enumerations.UsagePlan):
        raise TypeError("The usagePlan must be set to a class 'Enumerations.UsagePlan'.")
    if not isinstance(action, Enumerations.Action):
        raise TypeError("The action must be set to a class 'Enumerations.Action'.")
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    try:
        if action == Enumerations.Action.Action_Enable:
            if usagePlan == Enumerations.UsagePlan.UsagePlan_Administrator:
                response = client.create_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanAdminID', connection),
                    keyId=tokenId,
                    keyType='API_KEY'
                )
            elif usagePlan == Enumerations.UsagePlan.UsagePlan_Author:
                response = client.create_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanAuthorID', connection),
                    keyId=tokenId,
                    keyType='API_KEY'
                )
            elif usagePlan == Enumerations.UsagePlan.UsagePlan_Client:
                response = client.create_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanUserID', connection),
                    keyId=tokenId,
                    keyType='API_KEY'
                )
        elif action == Enumerations.Action.Action_Disable:
            if usagePlan == Enumerations.UsagePlan.UsagePlan_Administrator:
                response = client.delete_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanAdminID', connection),
                    keyId=tokenId
                )
            elif usagePlan == Enumerations.UsagePlan.UsagePlan_Author:
                response = client.delete_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanAuthorID', connection),
                    keyId=tokenId
                )
            elif usagePlan == Enumerations.UsagePlan.UsagePlan_Client:
                response = client.delete_usage_plan_key(
                    usagePlanId=Configuration.get('AWS_UsagePlanUserID', connection),
                    keyId=tokenId
                )
    except:
        #print('AmazonWebServices.apply : This action ( ' + action.serial + ' - ' + usagePlan.serial + ' ) has already been applied to this token.')
        pass
    # print(response) # Uncomment to print response

@staticmethod
def apply(tokenId, usagePlan, action, awsAccessKey, awsSecretKey, connection):
    if action == Enumerations.Action.Action_Enable:
        if usagePlan == Enumerations.UsagePlan.UsagePlan_Administrator:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Client, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Author, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Administrator, action, awsAccessKey, awsSecretKey, connection)
        elif usagePlan == Enumerations.UsagePlan.UsagePlan_Author:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Client, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Author, action, awsAccessKey, awsSecretKey, connection)
        elif usagePlan == Enumerations.UsagePlan.UsagePlan_Client:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Client, action, awsAccessKey, awsSecretKey, connection)
    elif action == Enumerations.Action.Action_Disable:
        if usagePlan == Enumerations.UsagePlan.UsagePlan_Administrator:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Administrator, action, awsAccessKey, awsSecretKey, connection)
        elif usagePlan == Enumerations.UsagePlan.UsagePlan_Author:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Author, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Administrator, action, awsAccessKey, awsSecretKey, connection)
        elif usagePlan == Enumerations.UsagePlan.UsagePlan_Client:
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Client, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Author, action, awsAccessKey, awsSecretKey, connection)
            __applySingleAction(tokenId, Enumerations.UsagePlan.UsagePlan_Administrator, action, awsAccessKey, awsSecretKey, connection)

def getUsagePlanKeys(usagePlan, awsAccessKey, awsSecretKey, connection):
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    if usagePlan == Enumerations.UsagePlan.UsagePlan_Client:
        plan = Configuration.get("AWS_UsagePlanUserID", connection)
    elif usagePlan == Enumerations.UsagePlan.UsagePlan_Author:
        plan = Configuration.get("AWS_UsagePlanAuthorID", connection)
    elif usagePlan == Enumerations.UsagePlan.UsagePlan_Administrator:
        plan = Configuration.get("AWS_UsagePlanAdminID", connection)
    response = client.get_usage_plan_keys(
        usagePlanId=plan,
        #position='1',
        limit=500,
        #nameQuery='a'
    )
    return response['items']

def getTokens(awsAccessKey, awsSecretKey):
    client = boto3.client('apigateway', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey,
                          region_name='us-west-2')
    response = client.get_api_keys(
        #position='string',
        limit=500,
        #nameQuery='string',
        #customerId='string',
        includeValues=True | False
    )
    return response['items']

def getTokenId(token, awsAccessKey, awsSecretKey):
    tokens = getTokens(awsAccessKey, awsSecretKey)
    print(tokens)
    tokenId = None
    for item in tokens:
        if item['value'] == token:
            tokenId = item['id']
    if tokenId != None:
        return tokenId
    else:
        raise KeyError('The token is not valid.')

def isUsagePlanPermitted(token, usagePlan, awsAccessKey, awsSecretKey, connection):
    permitted = False
    tokenId = getTokenId(token, awsAccessKey, awsSecretKey)
    keys = getUsagePlanKeys(usagePlan, awsAccessKey, awsSecretKey, connection)
    for key in keys:
        if key['id'] == tokenId:
            permitted = True
    return permitted


import typing as typ
import pandas as pd
import altair as alt

import pacifico_devel.util.reportTemplate.src.core.Plot.PlotAltair.PlotAltair as plta
import pacifico_devel.util.reportTemplate.src.core.Style.Style as stl
import pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors as pc

import pacifico_devel.util.lexikon.src.util.Enumerations as lenm
import pacifico_devel.util.lexikon.src.core.Translator as trl


class PlotAltairBar(plta.PlotAltair):
    def __init__(self,
                 df: pd.DataFrame,
                 shape: typ.Tuple[float, float],
                 languageInput: lenm.Language,
                 languageOutput: lenm.Language,
                 basePlotResolution: int,
                 style: typ.Optional[stl.Style] = None,
                 title: str = None):
        super(PlotAltairBar, self).__init__(df=df, shape=shape, languageInput=languageInput,
                                            languageOutput=languageOutput, basePlotResolution=basePlotResolution,
                                            style=style)
        self._title = title

    def _makeChart(self) -> alt.Chart:
        categoriesColumn = self.df.columns[0]
        valuesColumn = self.df.columns[1]

        self.df["barchart_color"] = self.df[valuesColumn].apply(self._colorMap)

        chart = alt.Chart(self.df).mark_bar().encode(x=valuesColumn,
                                                     y=alt.Y(categoriesColumn, sort="-x"),
                                                     color=alt.Color("barchart_color:N", scale=None))
        chart = chart.properties(title=self._title,
                                 width=self._shape[0],
                                 height=self._shape[1])
        return chart

    @staticmethod
    def _colorMap(x: float):
        if x > 0:
            return pc.PacificoPlotColors.BLUE.getCodeString()
        return pc.PacificoPlotColors.PINK.getCodeString()



import datetime as dt
import pathlib as pth
import typing as typ

from pacifico_devel.util.lexikon.src.util.Enumerations import Language


class Style:  # make abstract
    """
    Analogue to CSS?
    """
    _CSS_DEFAULT_PATH: pth.Path = pth.Path(__file__).parent.parent / "css" / "pacifico.css"

    def __init__(self,
                 cssPath: typ.Union[pth.Path, str] = _CSS_DEFAULT_PATH):
        self._cssPath = cssPath

    @staticmethod
    def _processIntegerPart(value: float,
                            language: Language) -> str:
        """
        Does formatting of integer part of float.

        Args:
            value:
            language:

        Returns:

        """
        absVal = abs(value)
        integer = str(int(absVal))

        reversedIntegerString = ""

        for i, char in enumerate(integer[::-1]):
            if i % 3 == 0 and i != 0:
                reversedIntegerString += language.thousandsSeparator()
            reversedIntegerString += char
        return reversedIntegerString[::-1]

    @staticmethod
    def _processDecimalPart(value: float,
                            numberDecimals: int) -> str:
        """
        Does processing of decimal part of float.

        Args:
            value:
            numberDecimals:

        Returns:

        """
        if numberDecimals < 0:
            raise ValueError(f"Number of decimals must be a positive integer, but is {numberDecimals}.")

        absVal = abs(value)
        if "." in str(absVal):
            decimalString: str = str(absVal).split(".")[-1]
        else:
            decimalString: str = ""

        if numberDecimals > len(decimalString):
            decimalString += "0" * (numberDecimals - len(decimalString))

        return decimalString[:numberDecimals]

    @classmethod
    def convertFloatToString(cls,
                             value: float,
                             numberDecimals: int,
                             language: Language) -> str:
        """
        Takes double and returns typical format of language. Ex: 1234.5678, EN -> 1,234.5678, [...]
        Use package that does this.

        Args:
            value:
            numberDecimals:
            language:

        Returns:

        """
        sign = "-" if value < 0 else ""

        integerPart = cls._processIntegerPart(value,
                                              language)

        if numberDecimals == 0:
            return sign + integerPart
        decimalPart = cls._processDecimalPart(value,
                                              numberDecimals)
        return sign + integerPart + language.decimalSeparator() + decimalPart

    @classmethod
    def convertIntToString(cls,
                           value: int,
                           language: Language) -> str:
        """
        Trivial. [excercise left to the reader]

        Args:
            value:
            language:

        Returns:

        """
        return cls.convertFloatToString(value=value,
                                        numberDecimals=0,
                                        language=language)

    @classmethod
    def convertDateTimeToString(cls,
                                value: dt.datetime,
                                language: Language) -> str:
        """
        Same same, but different (with dates). EN -> MM/DD/YYYY [...], LOCALE for hour.

        Args:
            value:
            language:

        Returns:

        """
        timeFormat = language.dateFormatString() + " %H:%M:%S"

        return value.strftime(timeFormat)

    @classmethod
    def convertDateToString(cls,
                            value: dt.date,
                            language: Language) -> str:
        """
        Same as previous.

        Args:
            value:
            language:

        Returns:

        """
        # return value.strftime(fmt="%d/%m/%Y")
        return value.strftime(language.dateFormatString())

    @classmethod
    def convertDateMonthToString(cls,
                                 value: dt.date,
                                 language: Language) -> str:
        """
        Either MM/YYYY or 'jun 2018'. For the moment, only first convention.

        Args:
            value:
            language:

        Returns:

        """
        formatString = language.dateSeparator().join(["%m", "%Y"])
        return value.strftime(formatString)

    @classmethod
    def convertDateQuarterToString(cls,
                                   value: dt.date,
                                   language: Language) -> str:
        """
        EN -> Q2/2018/.

        Args:
            value:
            language:

        Returns:

        """
        quarter = value.month // 3 + 1
        return "Q" + str(quarter) + language.dateSeparator() + str(value.year)

    @classmethod
    def convertDateYearToString(cls,
                                value: dt.date) -> str:
        return str(value.year)

    def getCss(self):
        """

        Returns: CSS corresponding to this style.

        """
        pass


if __name__ == '__main__':
    sty = Style()
    s = sty.convertFloatToString(value=1234567891.23,
                                 numberDecimals=1,
                                 language=Language.English_US)
    print(s)

    date = dt.date(year=2019, month=5, day=27)
    # ds = sty.convertDateMonthToString(date, language=Language.English_US)
    # ds = sty.convertDateQuarterToString(date, language=Language.English_US)
    ds = sty.convertDateToString(date, language=Language.German)

    for lang in Language:
        print(f"{lang.name}: {sty.convertDateToString(date, language=lang)}")

    # print(sty.convertDateToString(date, language=Language.German))
    # print(sty.convertDateToString(date, language=Language.English_US))



import string
import random
import numpy as np
import pandas as pd

import typing as typ


def _makeRandomString(length: int = 10):
    return "".join(random.choices(string.ascii_lowercase,
                                  k=length)
                   )


def makeRandomCategoricalDf(shape: typ.Tuple[int, int]):
    cols = [f"value_{i}" for i in range(shape[1])]
    idx = pd.date_range(start="2000-01-01", periods=shape[0])
    df = pd.DataFrame(np.random.randint(-100,
                                        100,
                                        size=shape),
                      columns=cols,
                      index=idx)
    df["category"] = [_makeRandomString(shape[0])
                      for _ in range(shape[0])]
    return df


if __name__ == '__main__':
    print(makeRandomCategoricalDf(shape=(20, 3)))
    print(makeRandomCategoricalDf(shape=(1, 3))["value_0"].values)



import abc
import pathlib as pth
import pandas as pd
import altair as alt

import pacifico_devel.util.reportTemplate.src.core.Plot.Plot as bplt


class PlotAltair(bplt.Plot, metaclass=abc.ABCMeta):
    """
    Abstract class for all necessary plots made with Altair Backend.
    """

    def saveImage(self) -> None:
        """


        Args:
            dfTranslated:

        Returns:

        """
        chart = self._makeChart()
        chart.save(fp=self.pathImage)

    @abc.abstractmethod
    def _makeChart(self) -> alt.Chart:
        """
        Makes chart.

        Args:
            dfTranslated:

        Returns:

        """
        pass

    def df_to_excel(self, writer, tag):
        """
        saves DataFrame to excel
        Returns:

        """

        self.df.to_excel(writer, sheet_name=tag)


"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

from ..util import Enumerations
import abc

class Object:
    def __init__(self):
        self.__id = None
        self.setObjectTypeValue()
        self.className = self.getClassName()

    def getClassName(self):
        return self.__class__.__name__

    def setObjectTypeValue(self):
        className = self.getClassName()
        try:
            self.objectTypeValue = Enumerations.ObjectType[className].value
        except:
            pass

    def setId(self, id):
        if isinstance(id, int):
            self.__id = id
        else:
            raise TypeError("The id must be set to a integer.")

    def getId(self):
        return self.__id

    def getObjectTypeValue(self):
        return self.objectTypeValue

    @staticmethod
    def fromJson(jsonString):
        pass

    @abc.abstractmethod
    def getJson(self):
        pass

    @abc.abstractmethod
    def saveToDB(self):
        pass


"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import datetime
from . import Field
from ....util import Enumerations, Dates
import json

class FieldTicker(Field.Field):
    def __init__(self, mnemo='', family='', group='', market='', country=Enumerations.Country.Country_Unspecified, other=''):
        self.mnemo = mnemo
        self.family = family
        self.group = group
        self.market = market
        self.country = country
        self.other = other

    # mnemo
    def _get_mnemo(self):
        return self.__mnemo
    def _set_mnemo(self, value):
        if not isinstance(value, str):
            raise TypeError("The mnemo must be set to a string.")
        self.__mnemo = value
    mnemo = property(_get_mnemo, _set_mnemo)

    # family
    def _get_family(self):
        return self.__family
    def _set_family(self, value):
        if not isinstance(value, str):
            raise TypeError("The family must be set to a string.")
        self.__family = value
    family = property(_get_family, _set_family)

    # group
    def _get_group(self):
        return self.__group
    def _set_group(self, value):
        if not isinstance(value, str):
            raise TypeError("The group must be set to a string.")
        self.__group = value
    group = property(_get_group, _set_group)

    # market
    def _get_market(self):
        return self.__market
    def _set_market(self, value):
        if not isinstance(value, str):
            raise TypeError("The market must be set to a string.")
        self.__market = value
    market = property(_get_market, _set_market)

    # country
    def _get_country(self):
        return self.__country
    def _set_country(self, value):
        if not isinstance(value, Enumerations.Country):
            raise TypeError("The country must be set to a class 'Enumerations.Country'.")
        self.__country = value
    country = property(_get_country, _set_country)

    # other
    def _get_other(self):
        return self.__other
    def _set_other(self, value):
        if not isinstance(value, str):
            raise TypeError("The other value must be set to a string.")
        self.__other = value
    other = property(_get_other, _set_other)

    # Class Methods

    def getMnemo(self):
        return self.mnemo

    def getFamily(self):
        return self.family

    def getGroup(self):
        return self.group

    def getMarket(self):
        return self.market

    def getCountry(self):
        return self.country

    def getOther(self):
        return self.other

    def getBase(self):
        for base in self.__class__.__bases__:
            return base.__name__

    @staticmethod
    def __serializeAndClean(dictionary):
        delete = []
        for key, value in zip(dictionary.keys(), dictionary.values()):
            if value in [[], {}, None, '']:
                delete.append(key)
            elif isinstance(value, list):
                dictionary[key] = [item.getJson() for item in value]
            elif isinstance(value, Enumerations.Enum):
                dictionary[key] = value.value
            elif isinstance(value, (datetime.date, datetime.datetime)):
                dictionary[key] = Dates.getDateOrDateTimeString(value)
            else:
                dictionary[key] = value
        for key in delete:
            del dictionary[key]
        return dictionary

    def getJson(self):
        # Get all attributes as a dictionary
        data = self.__dict__.copy()
        # Delete object private prefix from attribute names
        oldKeys = list(data.keys())
        for key in oldKeys:
            newKey = key.replace('_' + self.__class__.__name__ + '__', '')
            newKey = newKey.replace('_' + self.getBase() + '__', '')
            data[newKey] = data.pop(key)
        # Serialize object within the attribute and delete empty attributes
        data = self.__serializeAndClean(data)
        # jsonize data
        dataJson = json.dumps(data)
        return dataJson


import typing as typ
import deepl
import pandas as pd
import pandas.api.types as pdt
import re
import numpy as np
from datetime import datetime
import pacifico_devel.util.lexikon.src.util.Enumerations as lenm
import pacifico_devel.util.cfg.Configuration as cfg


class Translator:
    """
    Class that implements language translator (probably with deep learning).
    """
    _KEY_NAME = "deepl_translator_key"

    @classmethod
    def translate_dataframe(cls,
                            df: pd.DataFrame,
                            languageInput: lenm.Language,
                            languageOutput: lenm.Language) -> pd.DataFrame:
        """
        Translates all strings in dataframe.

        Args:
            df:
            languageInput:
            languageOutput:

        Returns:

        """
        dfAux = df.copy()
        dfAux = dfAux.applymap(lambda x: x.date() if isinstance(x, datetime) else x)
        ##
        if pdt.is_string_dtype(dfAux.index):
            indexes = cls.translate(text=dfAux.index,
                                    languageInput=languageInput,
                                    languageOutput=languageOutput)
            if isinstance(dfAux.index, pd.MultiIndex):
                dfAux.index = pd.MultiIndex.from_tuples(indexes)
            else:
                dfAux.index = indexes
        ##
        if dfAux.index.name is not None:
            #     dfAux.index.name = cls.translate(text="index",
            #                                      languageInput=lenm.Language.English_US,
            #                                      languageOutput=languageOutput)
            # else:
            dfAux.index.name = cls.translate(text=dfAux.index.name,
                                             languageInput=languageInput,
                                             languageOutput=languageOutput)
        ##
        if pdt.is_string_dtype(dfAux.columns):
            cols = cls.translate(text=dfAux.columns,
                                 languageInput=languageInput,
                                 languageOutput=languageOutput)
            if isinstance(dfAux.columns, pd.MultiIndex):
                dfAux.columns = pd.MultiIndex.from_tuples(cols)
            else:
                dfAux.columns = cols
        for col in dfAux.columns:
            if pdt.is_string_dtype(dfAux[col]):
                dfAux[col] = cls.translate(text=dfAux[col],
                                           languageInput=languageInput,
                                           languageOutput=languageOutput)
        if languageInput is not languageOutput:
            dfAux = dfAux.applymap(lambda x: float(x) if bool(re.match('^[-+]?[0-9]*\.?[0-9]+$', str(x))) else (
                np.nan if x == 'nan' else (
                    datetime.strptime(x, '%Y-%m-%d') if bool(re.search(r'(\d+-\d+-\d+)', str(x))) else x)))

        return dfAux

    @classmethod
    def translate(cls,
                  text: typ.Union[str, typ.Iterable[str]],
                  languageInput: lenm.Language,
                  languageOutput: lenm.Language) -> typ.Union[str, typ.List[str], typ.List[tuple]]:
        """

        Args:
            text:
            languageInput:
            languageOutput:

        Returns: translated text

        """

        translator = deepl.Translator(cls._getKey())
        print(f"SOURCE LANG: {languageInput.deeplStringOutput}")
        print(f"TARGET LANG: {languageOutput.deeplStringOutput}")
        if isinstance(text, str):
            if languageInput is languageOutput:
                return text
            result = translator.translate_text(text=text,
                                               source_lang=languageInput.deeplStringInput,
                                               target_lang=languageOutput.deeplStringOutput)
            return result.text
        elif hasattr(text, "__iter__"):
            if languageInput is languageOutput:
                return list(text)
            result = translator.translate_text(text=text,
                                               source_lang=languageInput.deeplStringInput,
                                               target_lang=languageOutput.deeplStringOutput)

            if isinstance(text, pd.MultiIndex):
                text = [item for t in text for item in t]
                result = [res.text if res.text != '' else t for res, t in zip(result, text)]
                return [*zip(result[::2], result[1::2])]
            result = [res.text if res.text != '' else t for res, t in zip(result, text)]
            return result

        raise ValueError(f"Invalid input: {text} of type {type(text)}. Must be of type str or an iterable of str.")

    @classmethod
    def _getKey(cls):
        """ Returns key from key-keeper"""
        return cfg.get(cls._KEY_NAME)


if __name__ == '__main__':
    text = """Aumentamos nuestra proyección de IPC de marzo desde 0.94 % hasta 1.08 % sin consecuencias relevantes para el acumulado del año. El seguimiento
    a productos como cigarrillos y gas por red nos llevó a adelantar la variación mensual que esperábamos ocurriera en el segundo trimestre. Para el caso
del precio del gas licuado empiezan a materializarse los riesgos al alza: desde la proyección inicial del mes preveíamos aumentos significativos, pero
el seguimiento de precios nos llevó a proyectar un incremento aún mayor. Todo lo anterior se vio parcialmente compensado por una mayor caída
proyectada para el pasaje en bus interurbano."""
    print(bool(re.match('^[-+]?[0-9]*\.?[0-9]+$', '2022-1-3')))
    # translation = Translator.translate(text=text,
    #                                    languageInput=lenm.Language.English_US,
    #                                    languageOutput=lenm.Language.Spanish)
    #
    # print(f"ORIGINAL TEXT: {text}")
    # print(f"TRANSLATED TEXT: {translation}")




import pandas as pd
import typing as typ
import pathlib as pth
import dataframe_image as dfi
import pandas.io.formats.style as pdst

import pacifico_devel.util.reportTemplate.src.core.Plot.Plot as bplt
# from pacifico_devel.util.reportTemplate.src.core.Plot.Table.Table import Table
# from pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors import PacificoPlotColors
import pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors as pc
# import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm
from pacifico_devel.util.reportTemplate.src.util.Patches import ScreenshotPatch

import pacifico_devel.util.reportTemplate.src.core.Plot.PlotAltair.PlotAltair as plta
import pacifico_devel.util.reportTemplate.src.core.Style.Style as stl
import pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors as pc

import pacifico_devel.util.lexikon.src.util.Enumerations as lenm
import pacifico_devel.util.lexikon.src.core.Translator as trl


# TO DO: Implement formatter

class PlotTableWithDivisions(bplt.Plot):
    def __init__(self,
                 df: pd.DataFrame,
                 languageInput: lenm.Language,
                 languageOutput: lenm.Language,
                 shape: typ.Tuple[float, float],
                 basePlotResolution: int,
                 style: typ.Optional[stl.Style] = None,
                 formatter: typ.Optional[typ.Dict[str, str]] = None,
                 columnsSubset: list = []):
        super(PlotTableWithDivisions, self).__init__(df=df,
                                                     shape=shape,
                                                     languageInput=languageInput,
                                                     languageOutput=languageOutput,
                                                     basePlotResolution=basePlotResolution,
                                                     style=style)
        self._formatter = formatter
        self.columnsSubset = columnsSubset

    def saveImage(self) -> None:
        """


        Args:
            dfTranslated:

        Returns:

        """
        chart = self._makeChart()
        ScreenshotPatch.patch()  # Monkey patch for chrome screenshots server
        print(f"SAVING TABLE TO: {self.pathImage} ...")
        dfi.export(obj=chart,
                   filename=str(self.pathImage))
        print(f"SAVING TABLE TO: {self.pathImage} ... DONE!")

    def _makeChart(self) -> pdst.Styler:
        # dfAux = self._translateDataFrame(df)
        thProps = [("background-color", pc.PacificoPlotColors.BLUE.getCodeString())]

        tdProps = [('font-family', 'Klartext Mono'),
                   ("color", "#330099"),
                   ('text-align', 'center'),
                   ('font-size', '14pt')]

        index_col = {'selector': 'th.col_heading',
                     'props': [('background-color', "#330099"),
                               ('font-family', 'Klartext Mono'),
                               ('color', 'white'),
                               ('text-align', 'center'),
                               ('font-size', '14pt')]}
        cols = []
        colHeadings = []
        for subset in self.columnsSubset:
            cols.append({"selector": ".col{}".format(subset),
                         "props": [('border-right', '5px solid #ffffff')]})
            colHeadings += [{'selector': 'th.col_heading.level{}.col{}'.format(level, subset),
                             'props': [('border-right', '5px solid #ffffff')]}
                            for level in range(3)]

        dfst = self.df.style.set_table_styles([{"selector": "th.blank", "props": thProps},
                                               {"selector": "td", "props": tdProps},
                                               index_col] + cols + colHeadings).hide_index()
        if self._formatter is None:
            dfst = dfst.format(lambda x: "{:.2f}".format(x) if isinstance(x, float) else '{}'.format(x), na_rep='')
        else:
            dfst = dfst.format(self._formatter, na_rep='')
        return dfst

    def df_to_excel(self, writer, tag):
        """
        saves DataFrame to excel
        Returns:

        """

        self.df.to_excel(writer, sheet_name=tag)


# from abc import ABCMeta, abstractmethod
import abc
import dataclasses
import pathlib as pth
# import bokeh.io as bio
import pandas as pd
# import pandas.api.types as pdt
import typing as typ
import dataclasses as dtc
# import dataframe_image as dfi
# import altair as alt
import chromedriver_autoinstaller

# import bokeh.plotting as bpl
# import bokeh.models as bm
# import altair.vegalite.v4.api as altapi
# import pandas.io.formats.style as pdst

from pacifico_devel.util.reportTemplate.src.core.Style.Style import Style
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm

# import pacifico_devel.util.lexikon.src.core.Translator as trl
import pacifico_devel.util.reportTemplate.src.util.Translation.DataFrame as tldf
import pacifico_devel.util.lexikon.src.util.Enumerations as lenm

from pacifico_devel.util.reportTemplate.src.util.Patches import ScreenshotPatch
import pacifico_devel.util.reportTemplate.src.util.CONFIG as CFG

import pacifico_devel.util.lexikon.src.core.Translator as trl


# MultiDisplay = typ.Union[bpl.Figure, bm.DataTable, altapi.Chart, pdst.Styler]

@dtc.dataclass(frozen=True)
class ImagePathHandler:
    """
    Class for handling image paths.
    """

    temp_root: pth.Path

    def makePathImage(self) -> pth.Path:
        idxMax = self._getMaximumDigit()

        if idxMax is None:
            plotIdx = 0
        else:
            plotIdx = idxMax + 1

        fileName = f"plot_{plotIdx}.png"
        return self.temp_root / fileName

    def _getMaximumDigit(self) -> typ.Optional[int]:
        digits = [self._digitFromFileName(filePath.stem)
                  for filePath in self.temp_root.glob("*.png")]
        digits = [x
                  for x in digits
                  if x is not None]
        if len(digits) > 0:
            return max(digits)
        return None

    @staticmethod
    def _digitFromFileName(s: str) -> typ.Optional[int]:
        base = s.split("_")[-1]
        if base.isnumeric():
            return int(base)
        return None


class Plot(metaclass=abc.ABCMeta):
    """
    Abstract class for all necessary plots
    """
    _IMAGE_PATH_HANDLER: ImagePathHandler = ImagePathHandler(CFG.ROOT_TEMP_IMG_PLOTS)
    _DATE_TIME_FORMAT = lenm.Language.English_US.dateFormatString()

    def __init__(self,
                 df: pd.DataFrame,
                 shape: typ.Tuple[float, float],
                 languageInput: lenm.Language,
                 languageOutput: lenm.Language,
                 basePlotResolution: int,
                 style: typ.Optional[Style] = None):
        self._df = trl.Translator.translate_dataframe(df=df,
                                                      languageInput=languageInput,
                                                      languageOutput=languageOutput)
        self._base_resolution = basePlotResolution

        self._shape = tuple(int(dim * self._base_resolution)
                            for dim in shape
                            )  # might be changed # TO DO: adapt to bokeh as well.
        self._languageInput = languageInput
        self._languageOutput = languageOutput
        self._style = style
        self._pathImage = self._IMAGE_PATH_HANDLER.makePathImage()  # self._makePathImage()

    @property
    def pathImage(self) -> pth.Path:
        return self._pathImage

    @property
    def df(self) -> pd.DataFrame:
        """

        Returns: dataframe of plot, translated.

        """
        return self._df

    def deleteImage(self) -> None:
        """
        Deletes plot image file.

        Returns:

        """
        self._pathImage.unlink()



    # def makeImage(self) -> pth.Path:
    #     """
    #     Translates dataframe and saves image of chart from dataframe. Finally, saves path to image
    #
    #     Returns:
    #
    #     """
    #     self._saveImage(self._pathImage)
    #     return self._pathImage

    @abc.abstractmethod
    def saveImage(self) -> None:
        """
        Saves image of chart from dataframe.

        Returns:

        """
        pass

    @abc.abstractmethod
    def df_to_excel(self, writer, tag):
        """
        saves DataFrame to excel
        Returns:

        """
        pass

    # def set_in_report(self,
    #                   report,
    #                   tag,
    #                   shape) -> None:
    #     self.saveImage()
    #     report.setImage(tag=tag,
    #                     path=self.pathImage,
    #                     width=shape[0],
    #                     height=shape[1])
    #     report.set_temp_plot(tag, self)


if __name__ == '__main__':
    print("/n")
    p = pth.Path(__file__).parent / "tmp"
    print(p)
    for x in p.glob("*.png"):
        print(x)

    # p = Plot(...)
    #
    # data = p.df
    #
    # new_data = get_new_data(...)
    # p.df = new_data



"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import datetime
import json
import abc
from pacifico.core.Service import Service

class ServiceRequest(Service.Service):

    @abc.abstractmethod
    def getJson(self):
        pass

    @staticmethod
    def fromJson(json):
        pass

    @staticmethod
    def __serializeAndClean(dictionary):
        delete = []
        for key, value in zip(dictionary.keys(), dictionary.values()):
            if value == [] or value == {} or value == None:
                delete.append(key)
            elif isinstance(value, (datetime.date, datetime.datetime)):
                dictionary[key] = value.isoformat()
            elif isinstance(value, dict):
                dictionary[key] = ServiceRequest.__serializeAndClean(value)
            elif isinstance(value, list):
                try:
                    dictionary[key] = [item.getJson() for item in value]
                except:
                    dictionary[key] = json.dumps(value)
            else:
                dictionary[key] = value
        for key in delete:
            del dictionary[key]
        return dictionary

    def getJson(self):
        # Get all attributes as a dictionary
        data = self.__dict__.copy()
        # Delete object private prefix from attribute names
        oldKeys = list(data.keys())
        for key in oldKeys:
            newKey = key.replace('_' + self.getClassName() + '__', '')
            newKey = newKey.replace('_' + self.getBase() + '__', '')
            data[newKey] = data.pop(key)
        # Serialize object within the attribute and delete empty attributes
        data = self.__serializeAndClean(data)
        # jsonize data
        dataJson = json.dumps(data)
        return dataJson

    def getClassName(self):
        return self.__class__.__name__

    def getBase(self):
        for base in self.__class__.__bases__:
            return base.__name__